﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;

namespace ProjectCreation
{
    class Program
    {
        static void Main(string[] args)
        {
            const string ProjectPath = @"c:\book\chapter19\Project.ispac";
            
            PackageEvents events = new PackageEvents();

            //
            // Create a new project with a default storage path
            //
            using (Project project = Project.CreateProject(ProjectPath))
            {
                // Set events
                project.Events = events;

                // Add a project parameter
                Parameter projectParam = project.Parameters.Add("BatchSize", TypeCode.Int32);
                projectParam.Value = 1000;

                // Add a shared connection manager
                var cm = project.ConnectionManagerItems.Add("OLEDB", "MyProjConnection.conMgr");
                cm.ConnectionManager.Name = "MyProjConnection";
                cm.ConnectionManager.ConnectionString = @"Data Source=.;Initial Catalog=AdventureWorksDW;Provider=SQLNCLI11.1;Integrated Security=SSPI;";

                // Create a package with a parameter 
                Package package = new Package();
                package.Name = "Package1";

                Parameter packageParam = package.Parameters.Add("InputFileName", TypeCode.String);
                packageParam.Value = @"\\data\inputfile.txt";

                // Add the new package to the project
                project.PackageItems.Add(package, "Package1.dtsx");

                // Save the project
                project.Save();
            }

            //
            // Reload the project, and run a package
            //
            using (Project project = Project.OpenProject(ProjectPath))
            {
                project.Events = events;

                // change the project parameter value for this run
                project.Parameters["BatchSize"].Value = 5000;

                // get a reference to the package we want to run
                var package = project.PackageItems["Package1.dtsx"].Package;

                package.Execute(null, null, events, null, null);

                if (package.Errors.Count > 0)
                {
                    Console.WriteLine("Package failed!");
                }
            }
        }
    }
}
