﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.SqlServer.Dts.Runtime;
using System.Data.SqlClient;
using System.Collections.ObjectModel;

namespace PackageRunner
{
    class PackageRunner
    {
        const string ProjectPath = @"F:\SSIS Inside Out\Chapter 6\SimpleProject\bin\Development\SimpleProject.ispac";
        const string ConnectionString = @"Data Source=localhost;Initial Catalog=Chapter6;Integrated Security=SSPI;";

        class WorkItemExecution
        {
            public Int64 WorkItemId { get; set; }
            public string PackageName { get; set; }
            public string InputFileName { get; set; }
        }

        static void Main(string[] args)
        {
            var events = new PackageEvents();

            // Load project
            using (var project = Project.OpenProject(ProjectPath, Project.AccessMode.Read, null, events))
            {
                // Get list of work items
                using (var connection = new SqlConnection(ConnectionString))
                {
                    connection.Open();

                    var workItems = new Collection<WorkItemExecution>();

                    var cmd = new SqlCommand("SELECT WorkItemID, PackageName, InputFileName " +
                                             "FROM WorkItems " +
                                             "WHERE WorkItemState = N'Queued'", connection);
                    using (var reader = cmd.ExecuteReader())
                    {
                        while (reader.Read())
                        {
                            var item = new WorkItemExecution()
                            {
                                WorkItemId = reader.GetInt64(0),
                                PackageName = reader.GetString(1),
                                InputFileName = reader.GetString(2)
                            };

                            workItems.Add(item);
                        }
                    }

                    foreach (var item in workItems)
                    {
                        // Get package
                        var packageItem = project.PackageItems[item.PackageName];
                        var package = packageItem.Package;

                        // Set parameter value
                        package.Parameters["InputFileName"].Value = item.InputFileName;

                        // Change row state
                        var updateCmd = new SqlCommand("UPDATE WorkItems " + 
                                                       "SET WorkItemState = N'Running', " +
                                                       "    LastUpdate = GETDATE() " +
                                                       "WHERE WorkItemID = @workItemId", connection);
                        updateCmd.Parameters.AddWithValue("workItemId", item.WorkItemId);
                        updateCmd.ExecuteNonQuery();

                        // Run package
                        package.Execute(null, null, events, null, null);

                        // Check for errors
                        if (package.Errors.Count > 0)
                        {
                            var failedCmd = new SqlCommand("UPDATE WorkItems " +
                                                            "SET WorkItemState = N'Failed', " +
                                                            "    LastUpdate = GETDATE() " +
                                                            "WHERE WorkItemID = @workItemId", connection);
                            failedCmd.Parameters.AddWithValue("workItemId", item.WorkItemId);
                            failedCmd.ExecuteNonQuery();
                        }
                        else
                        {
                            var successCmd = new SqlCommand("UPDATE WorkItems " +
                                                            "SET WorkItemState = N'Succeeded', " +
                                                            "    LastUpdate = GETDATE() " +
                                                            "WHERE WorkItemID = @workItemId", connection);
                            successCmd.Parameters.AddWithValue("workItemId", item.WorkItemId);
                            successCmd.ExecuteNonQuery();
                        }
                    }
                }
            }
        }


        internal class PackageEvents : DefaultEvents
        {
            public override bool OnError(DtsObject source, int errorCode, string subComponent, 
                                         string description, string helpFile, int helpContext, 
                                         string idofInterfaceWithError)
            {
                Console.WriteLine("[Error] {0}", description);
                return base.OnError(source, errorCode, subComponent, description, helpFile, 
                                    helpContext, idofInterfaceWithError);
            }
        }
    }
}
